/********************************************************************
  *
  *  interface   : AbstractUserAdministration.java
  *  author      : Helge Janicke, Niels-Peter de Witt
  *  version     : 2000-11-28(001)
  *  (c)         : Helge Janicke, Niels-Peter de Witt
  *  email       : heljanic@agent.fho-emden.de, ndewitt@gmx.de
  *  description : Handles the user administration with a database.
  *
  *******************************************************************
  * modifications:
  * date: ******* who: ****** description: **************************
  * -----------------------------------------------------------------
  * 2000-10-19    heljanic    your description here
  * -----------------------------------------------------------------
  *******************************************************************
  * comments:
  *******************************************************************
  * license:
	********************************************************************/

package de.janicke.hj.tools.servlet;

import java.sql.*;


/** Handles the user administration with a database.
	*/
public abstract class AbstractUserAdministration {

/** Generates a SecureIdentity String with n chars.
	* @param n Length of the generated String.
	* @return A randomly choosen String with a length of n.
	*/
  public static String getRandomSecureId(int n) {
    java.util.Random rnd = new java.util.Random();
    StringBuffer sb = new StringBuffer();
    for (int i=0; i<n; i++) {
      sb.append( (char) (rnd.nextInt(200) + 32));
    }
    return sb.toString();
  }


// ------------------------------------------------------------------------------- add User

	/** Adds a user to the database.
		*	@param user The user to add.
		* @return <tt>true</tt> if the adding succesfull, else <tt>false</tt>.
		*/
  public abstract boolean addUser(User user);

// ------------------------------------------------------------------------------- accept User

	/** Returns if a user exists in the database.
		*	@param user The user to check with database.
		*	@return <tt>true</tt> if the user exist, else <tt>false</tt>.
		*/
  public boolean acceptUser(User user) {
    return acceptUser( user.getId(), user.getSecureIdentity() );
  }

	/** Returns if a user exists in the database.
		*	@param id The userid to check with database.
		*	@param sid The secureIdentity of the user to check with database.
		*	@return <tt>true</tt> if the user exist, else <tt>false</tt>.
		*/
  public abstract boolean acceptUser(int id, String sid);

  /** Returns if a user exists in the database.
		*	@param nick The nick of the user to check with database.
		*	@param passwd The password of the user to check with database.
		*	@return <tt>true</tt> if the user exist, else <tt>false</tt>.
		*/
  public abstract boolean acceptUser(String nick, String passwd);

 	/** Returns if a user exists in the database.
		*	@param id The <tt>userid</tt> to check with database.
		*	@return <tt>true</tt> if the user exist, else <tt>false</tt>.
		*/
  public abstract boolean acceptUser(int id);

  /** Returns if a user exists in the database.
		*	@param nick The nick of the user to check with database.
		*	@return <tt>true</tt> if the user exist, else <tt>false</tt>.
		*/
  public abstract boolean acceptUser(String nick);

// ------------------------------------------------------------------------------- delete User

	/** Deletes an existing user from the database.
		*	@param user The user to delete.
		*	@return <tt>true</tt> if the user exist and was successfully deleted,
		* if user does not exists it returns <tt>false</tt>.
		*	@throws Exception if user exists, but cannot be delted from database.
		*/
  public boolean deleteUser(User user) throws Exception {
    if (acceptUser(user)) {
      return deleteUser( user.getNickname(), user.getPassword() );
    }
    return false;
  }

  /** Deletes an existing user from the database.
		*	@param nick The nickname of the user to delete.
		*	@param passwd The password of the user to delete.
		*	@return <tt>true</tt> if the user exist and was successfully deleted, if user does not exists it returns <tt>false</tt>.
		*	@throws Exception if user exists, but cannot be delted from database.
		*/
  public abstract boolean deleteUser(String nick, String passwd) throws Exception;

 	/** Deletes an existing user from the database.
		*	@param id The <tt>userid</tt> to delete.
		*	@return <tt>true</tt> if the user exist and was successfully deleted, if user does not exists it returns <tt>false</tt>.
		*	@throws Exception if user exists, but cannot be delted from database.
		*/
  public abstract boolean deleteUser(int id) throws Exception;

// ------------------------------------------------------------------------------- edit User

	/** Edits an existing user from the database.
		*	@param user The user to edit.
		*	@return If the user exist and was successfully updated the old user will be returned.
		* @throws SQLException if database cannot be reached or problems with the SQL statements.
		*	@throws Exception if user does not exists.
		*/
  public abstract User editUser(User user) throws Exception;

	/** Edits an existing user password from the database.
		*	@param user The user to edit.
		* @param passwd The new password.
		*	@return If the user exist and password was successfully updated the old user will be returned.
		* @throws SQLException if database cannot be reached or problems with the SQL statements.
		*	@throws Exception if user does not exists.
		*/
  public abstract User editUser(User user, String passwd) throws Exception;

// ------------------------------------------------------------------------------- login User

	/**	Tries to login a new user.
		*	@param user The user who wants to login
		* @return It returns the existing user object from the database if it exists, else <tt>null</tt>.
		* @throws Exception if user exists but secureIdentity could not be set.
		*/
  public User loginUser(User user) throws Exception {
    if ( !acceptUser( user.getId(), user.getSecureIdentity() ) ) {
      return loginUser( user.getNickname(), user.getPassword() );
    } else throw new Exception("You are already logged in.");
  }

	/**	Tries to login a new user.
		*	@param nick The nickname of the user who wants to login
		*	@param passwd The password of the user who wants to login
		* @return It returns the existing user object from the database if it exists, else <tt>null</tt>.
		* @throws Exception if user exists but secureIdentity could not be set.
		*/
  public abstract User loginUser(String nick, String passwd) throws Exception;

// ------------------------------------------------------------------------------- get User

	/** Gets a user from the database.
		* @param id The id of the user to get.
		* @return The user from the database chooesen by the id, if not exists it returns <tt>null</tt>.
		* @throws SQLException if there is no conection to the database or problems with the SQL statements.
		*/
  public abstract User getUser(int id) throws SQLException;

	/** Gets a user from the database.
		* @param nick The nickname of the user to get.
		* @return The user from the database choosen by the nickname, if not exists it returns <tt>null</tt>.
		* @throws SQLException if there is no conection to the database or problems with the SQL statements.
		*/
  public abstract User getUser(String nick) throws SQLException;

	/** Gets a user from the database.
		* @param id The id of the user to get.
		* @param sid The secureIdentity of the user to get,
		* @return The user from the database choosen by the id and secureIdentity, if not exists it returns <tt>null</tt>.
		* @throws SQLException if there is no conection to the database or problems with the SQL statements.
		*/
  public abstract User getUser(int id, String sid) throws SQLException;

  /** Gets a user from the database.
		* @param nick The nickname of the user to get.
		* @param passwd The password of the user to get.
		* @return The user from the database choosen by the nickname and password, if not exists it returns <tt>null</tt>.
		* @throws SQLException if there is no conection to the database or problems with the SQL statements.
		*/
  public abstract User getUser(String nick, String passwd) throws SQLException;

}