/********************************************************************
  *
  *  program     : Database.java
  *  author      : Helge Janicke
  *  version     : 2000-11-28(001)
  *  (c)         : Helge Janicke
  *  email       : heljanic@agent.fho-emden.de
  *  description : This class handles all SQL interchanges.
  *
  *******************************************************************
  * modifications:
  * date: ******* who: ****** description: **************************
  * -----------------------------------------------------------------
  * 2000-10-19    heljanic    your description here
  * -----------------------------------------------------------------
  *******************************************************************
  * comments:
  *******************************************************************
  * license:
********************************************************************/

package de.janicke.hj.tools.servlet;

import java.sql.*;

/** Class handles the database connection. There is functionality to
  * precompile SQL statements.<br>
  * jdbc stands for JavaDataBaseConnection.
  */
public class Database {

	/** DB-Connection */
	public Connection jdbc;

	/** String that identifies the driver class. */
	private String jdbcDriver;

	/** String that identifies the database connection. */
	private String jdbcConnection;

	/** indicates if the connection is established. */
	private boolean opened = false;


//------------------------------------------------------- constructors

	/** Constructs a new Database object.
	  * @param jdbcDriver String that identifies the driver class.
	  * @param jdbcConnection String that identifies the database
	  *		 	connection.
	  */
	public Database(String jdbcDriver, String jdbcConnection) {

		this.jdbcDriver = jdbcDriver;
		this.jdbcConnection = jdbcConnection;
	}

	public Database() {
		this.jdbcDriver = "sun.jdbc.odbc.JdbcOdbcDriver";
		this.jdbcConnection = "jdbc:odbc:VocTrainerDB";
	}



//------------------------------------------------------------ methods

	/** Opens the database specified by the jdbcConnection value.
	  * Checks also for existance of the driver class specified by
	  * jdbcDriver.
	  * @return true if succeded, false otherwise.
	  */
	public boolean open () {
		try {
			Class.forName(jdbcDriver);
			jdbc = DriverManager.getConnection(jdbcConnection);
			jdbc.setAutoCommit(false);
		} catch (SQLException exc) {
			System.err.println("Tried to open the Database connection\n" +
							   "Unable to build up the connection.");
			exc.printStackTrace();
			return false;
		} catch (ClassNotFoundException exc) {
			System.err.println("Tried to open the Database connection\n" +
							   "Driver class specified by " + jdbcDriver
							   + " not found.");
			exc.printStackTrace();
			return false;
		}
		opened = true;
		return true;
	}
	
	/** Makes all changes made since the previous commit/rollback permanent and releases any database locks
    * currently held by the Connection.
    */
	public void commitData() {
		try {
			jdbc.commit();
		} catch (SQLException sql_exc)
		{ sql_exc.printStackTrace();
		}
	}

	/** Closes the database connection.
	  * @return true if succeded, false otherwise.
	  */
	public boolean close () {
		try	{
			jdbc.close();
		}
		catch (SQLException exc) {
			System.err.println("Tried to close database connection.\n");
			exc.printStackTrace();
			return false;
		}
		opened = false;
		return true;
	}

	/** Returns true if connection established.
	  * @return true if opened.
	  */
	public boolean isOpened() {
		return this.opened;
	}


	/** Precompiles a SQL statement.
	  * @param sqlStatement The SQL statement to compile.
	  * @return The precompiled SQL statement, or null if the
	  * 		connection is closed.
	  */
	public PreparedStatement precompile(String sqlStatement)
				throws SQLException {
		if (!isOpened()) return null;
		return jdbc.prepareStatement(sqlStatement);
	}

}