/********************************************************************
  *
  *  program     : HTMLTemplate.java
  *  author      : Helge Janicke
  *  version     : 2000-11-30(002)
  *  (c)         : Helge Janicke
  *  email       : heljanic@agent.fho-emden.de
  *  description : Replaces variable definitions in a HTML template
  *				   file with the values that are specified in a
  *				   HashMap.
  *
  *******************************************************************
  * modifications:
  * date: ******* who: ****** description: **************************
  * -----------------------------------------------------------------
  * 2000-11-30    heljanic    modified thrown Exeptions in
  *							  buildTemplate. See documentation.
  * -----------------------------------------------------------------
  *******************************************************************
  * comments:
  *******************************************************************
  * license:
********************************************************************/
package de.janicke.hj.tools.servlet;

import java.util.*;

import javax.servlet.*;
import javax.servlet.http.*;
import java.io.File;


public class ServletParameters {

	/** Der Name der ServletKlasse.  */
	private	String className = null;

	/** ServletName<br>
	Wenn Parameter(ServletName) nicht angegeben, dann ' "/" + className '.*/
	private	String servletName = null;

	/** Der Path auf dem sich das Servlet auf der Festplatte befindet. */
	private	java.io.File servletPath;

	/** Der Path auf dem sich die Templates auf der Festplatte befinden.<br>
	Wenn Parameter(TemplatePath) nicht angegeben, dann ' m_strServletPath + "templates" + fileSeparator '.*/
	private java.io.File templatePath = null;

	/** Die URL, wo die Images liegen.<br>
	Wenn Parameter(ImagesURL) nicht angegeben, dann ' servletName + "/images/" '.<br>
	Wird eine ImagesURL angegeben, die nicht auf dem localen Server liegt, muss diese mit 'http://' beginnen. */
	private String imageURL = null;

	// -------------------- Construktor ---------------
	public ServletParameters ( GenericServlet gs ) {
	  try {
		init(gs);
	  } catch (Exception exc) {
		System.err.println( exc.getMessage() );
		exc.printStackTrace();
	  }

	}

	private void init(GenericServlet gs) throws Exception {

	  // Get the classname.
	  String  tempClassName = gs.getClass().getName();
	  this.className = tempClassName.substring(tempClassName.lastIndexOf('.') + 1);

	  // Get the servletname.
	  this.servletName = gs.getInitParameter( "ServletName" );
	  if (this.servletName == null) this.servletName = "/" + className;

      // Get the servlet path
	  String tempServletPath = gs.getServletContext().getRealPath(servletName);
	  if (tempServletPath != null) {
		servletPath = new java.io.File(tempServletPath);
	  } else {
		throw new NullPointerException("GenericServlet.init(): No real path specified for servlet: " + servletName);
	  }

	  // Get the template path.
	  String tempTemplatePath = gs.getInitParameter( "TemplatePath" );
	  if (tempTemplatePath == null)
		  this.templatePath = new File (this.servletPath.getParent(), "templates");

	  // Get the URL for the images.
	  imageURL = gs.getInitParameter( "ImagesURL" );
	  if (imageURL == null) imageURL = servletName + "/images/";
	}


	/** Returns the class name of the servlet.
	  * @return The class name.
	  */
	public String getClassName() {
	  return this.className;
	}

	/** Returns the name of the servlet.
	  * @return The servlet name.
	  */
	public String getServletName() {
	  return this.servletName;
	}

	/** Returns the real path of the servlet.
	  * @return The servlet's real path.
	  */
	public java.io.File getServletPath() {
	  return this.servletPath;
	}

	/** Returns the path of the servlet's template files.
	  * @return The template path.
	  */
	public java.io.File getTemplatePath() {
	  return this.templatePath;
	}

	/** Returns the URL of the images.
	  * @return The image URL.
	  */
	public String getImageURL() {
	  return this.imageURL;
	}

}
