/********************************************************************
  *
  *  interface   : SimpleUserAdministration.java
  *  author      : Helge Janicke, Niels-Peter de Witt
  *  version     : 2000-11-28(001)
  *  (c)         : Helge Janicke, Niels-Peter de Witt
  *  email       : heljanic@agent.fho-emden.de, ndewitt@gmx.de
  *  description : Handles the user administration with a database.
  *
  *******************************************************************
  * modifications:
  * date: ******* who: ****** description: **************************
  * -----------------------------------------------------------------
  * 2000-10-19    heljanic    your description here
  * -----------------------------------------------------------------
  *******************************************************************
  * comments:
  *******************************************************************
  * license:
	********************************************************************/
	
package de.janicke.hj.tools.servlet;

import java.sql.*;
import javax.servlet.http.*;

/** Handles the user administration with a database.
	*/
public class SimpleUserAdministration extends AbstractUserAdministration {

  protected Database database = null;

  //  adduser
  private final String sqlAddUser=
    "INSERT INTO user (nickname, password, forename, surename, email ) "
    + "VALUES ( ? ,?, ?, ?, ? ); ";

  //  deluser
  private final String sqlDeleteUserById= "DELETE * "
    + "FROM user "
    + "WHERE id = ? ; ";

  private final String sqlDeleteUserByNick_Passwd= "DELETE * "
    + "FROM user "
    + "WHERE nickname = ? AND password = ? ; ";

  // accept id
  private final String sqlAcceptUserById= "SELECT id FROM user WHERE id = ? ; ";

  private final String sqlAcceptUserById_Sid= "SELECT id FROM user WHERE id = ? AND secureidentity = ? ; ";

  private final String sqlAcceptUserByNick_Passwd= "SELECT id FROM user WHERE nickname = ? AND password = ? ; ";

  private final String sqlAcceptUserByNick= "SELECT id FROM user WHERE nickname = ? ; ";

  // edit user
  private final String sqlEditUser= "UPDATE user SET nickname = ? , forename = ? , surename = ? , email = ? " +
    "WHERE id = ? ; ";

  private final String sqlEditUserPwd= "UPDATE user SET password = ? " +
    "WHERE id = ? AND password = ? ; ";

  private final String sqlGetUserById= "SELECT id , nickname , password , forename , surename , email , secureidentity , suflag " +
    "FROM user WHERE id = ? ; ";

  private final String sqlGetUserByNick= "SELECT id , nickname , password , forename , surename , email , secureidentity , suflag " +
    "FROM user WHERE nickname = ? ; ";

  private final String sqlGetUserByNick_Passwd= "SELECT id , nickname , password , forename , surename , email , secureidentity , suflag " +
    "FROM user WHERE nickname = ? AND password = ? ; ";

  private final String sqlGetUserById_Sid= "SELECT id , nickname , password , forename , surename , email , secureidentity , suflag " +
    "FROM user WHERE id = ? AND secureidentity = ? ; ";

  // SQL to set SecureId by ID
  private final String sqlSetSecureIdentity =
  "UPDATE user SET secureIdentity = ? WHERE id = ?;";

  protected PreparedStatement preSQLSetSecureIdentity;

  /** Precompiled SQL statement to add a servlet user. */
  protected PreparedStatement preSQLAddUser;

  /** Precompiled SQL statement to delete a servlet user. */
  protected PreparedStatement preSQLDeleteUserByNick_Passwd;

  /** Precompiled SQL statement to delete a servlet user. */
  protected PreparedStatement preSQLDeleteUserById;

  /** Precompiled SQL statement to get a servlet user. */
  protected PreparedStatement preSQLGetUserById;

  /** Precompiled SQL statement to get a servlet user. */
  protected PreparedStatement preSQLGetUserByNick;

  /** Precompiled SQL statement to get a servlet user. */
  protected PreparedStatement preSQLGetUserByNick_Passwd;

  /** Precompiled SQL statement to get a servlet user. */
  protected PreparedStatement preSQLGetUserById_Sid;

  protected PreparedStatement preSQLAcceptUserById;

  protected PreparedStatement preSQLAcceptUserById_Sid;

  protected PreparedStatement preSQLAcceptUserByNick_Passwd;

  protected PreparedStatement preSQLAcceptUserByNick;

  protected PreparedStatement preSQLEditUser;

  protected PreparedStatement preSQLEditUserPwd;

	/** Sets the database for the UserAdministration.
		* @param db The used Database.
		*/
  public SimpleUserAdministration(Database db) {
    this.database = db;
  }

// ----------------------------------------------------------------------------------- Init  
  
	/** Initializes the database.
	  * compiles SQL statements.
	  * @throws Exception if there is no connect to the database, or the precompile of SQL statements fails.
	  */
  public void init() throws Exception {
    if (!database.isOpened()) throw new Exception("database not opened. Please open database first before calling init()!");

    preSQLAddUser                 =  database.precompile(sqlAddUser);
    preSQLDeleteUserByNick_Passwd =  database.precompile(sqlDeleteUserByNick_Passwd);
    preSQLDeleteUserById          =  database.precompile(sqlDeleteUserById);
    preSQLGetUserById             =  database.precompile(sqlGetUserById);
    preSQLGetUserByNick           =  database.precompile(sqlGetUserByNick);
    preSQLGetUserByNick_Passwd    =  database.precompile(sqlGetUserByNick_Passwd);
    preSQLGetUserById_Sid         =  database.precompile(sqlGetUserById_Sid);
    preSQLEditUser                =  database.precompile(sqlEditUser);
    preSQLEditUserPwd             =  database.precompile(sqlEditUserPwd);

    preSQLAcceptUserById          =  database.precompile(sqlAcceptUserById);
    preSQLAcceptUserById_Sid      =  database.precompile(sqlAcceptUserById_Sid);
    preSQLAcceptUserByNick_Passwd =  database.precompile(sqlAcceptUserByNick_Passwd);
    preSQLAcceptUserByNick        =  database.precompile(sqlAcceptUserByNick);

    preSQLSetSecureIdentity =  database.precompile(sqlSetSecureIdentity);

  }

// ------------------------------------------------------------------------------- add User

	/** Adds a user to the database.
		*	@param user The user to add.
		* @return <tt>true</tt> if the adding succesfull, else <tt>false</tt>.
		*/
  public boolean addUser(User user) {

    if (acceptUser(user.getNickname())) return false;

    try {
      preSQLAddUser.clearParameters();
      preSQLAddUser.setString(1, user.getNickname());
      preSQLAddUser.setString(2, user.getPassword());
      preSQLAddUser.setString(3, user.getForename());
      preSQLAddUser.setString(4, user.getSurename());
      preSQLAddUser.setString(5, user.getEmail());
      preSQLAddUser.executeUpdate();
      database.commitData();
      return true;
    } catch (SQLException exc) {
      exc.printStackTrace();
    }
    return false;
  }

// ------------------------------------------------------------------------------- accept User  
  
	/** Returns if a user exists in the database.
		*	@param nick The nick of the user to check with database.
		*	@return <tt>true</tt> if the user exist, else <tt>false</tt>.
		*/  
  public boolean acceptUser(String nick) {
    try {
      preSQLAcceptUserByNick.clearParameters();
      preSQLAcceptUserByNick.setString(1, nick);
      ResultSet result = preSQLAcceptUserByNick.executeQuery();
      if( result.next() ) {
        // Nickname exists.
        result.close();
        return true;
      } else {
        result.close();
      }
    } catch (SQLException exc) {
      exc.printStackTrace();
    }
    return false;
  }

	/** Returns if a user exists in the database.
		*	@param id The userid to check with database.
		*	@param sid The secureIdentity of the user to check with database.
		*	@return <tt>true</tt> if the user exist, else <tt>false</tt>.
		*/  
  public boolean acceptUser(int id, String sid) {
    try {
      preSQLAcceptUserById_Sid.clearParameters();
      preSQLAcceptUserById_Sid.setInt(1, id);
      preSQLAcceptUserById_Sid.setString(2, sid);
      ResultSet result = preSQLAcceptUserById_Sid.executeQuery();
      if( result.next() ) {
        // id and sid exists.
        result.close();
        return true;
      } else {
        result.close();
      }
    } catch (SQLException exc) {
      exc.printStackTrace();
    }
    return false;
  }

	/** Returns if a user exists in the database.
		*	@param nick The nick of the user to check with database.
		*	@param passwd The password of the user to check with database.	
		*	@return <tt>true</tt> if the user exist, else <tt>false</tt>.
		*/  
  public boolean acceptUser(String nick, String passwd) {
    try {
      preSQLAcceptUserByNick_Passwd.clearParameters();
      preSQLAcceptUserByNick_Passwd.setString(1, nick);
      preSQLAcceptUserByNick_Passwd.setString(2, passwd);
      ResultSet result = preSQLAcceptUserByNick_Passwd.executeQuery();
      if( result.next() ) {
        // nick and passwd exists.
        result.close();
        return true;
      } else {
        result.close();
      }
    } catch (SQLException exc) {
      exc.printStackTrace();
    }
    return false;
  }

 	/** Returns if a user exists in the database.
		*	@param id The <tt>userid</tt> to check with database.
		*	@return <tt>true</tt> if the user exist, else <tt>false</tt>.
		*/  
  public boolean acceptUser(int id) {
    try {
      preSQLAcceptUserById.clearParameters();
      preSQLAcceptUserById.setInt(1, id);
      ResultSet result = preSQLAcceptUserById.executeQuery();
      if( result.next() ) {
        // id exists.
        result.close();
        return true;
      } else {
        result.close();
      }
    } catch (SQLException exc) {
      exc.printStackTrace();
    }
    return false;
  }

// ------------------------------------------------------------------------------- delete User

  /** Deletes an existing user from the database.
		*	@param nick The nickname of the user to delete.
		*	@param passwd The password of the user to delete.
		*	@return <tt>true</tt> if the user exist and was successfully deleted, if user does not exists it returns <tt>false</tt>.
		*	@throws Exception if user exists, but cannot be delted from database.
		*/  
  public boolean deleteUser(String nick, String passwd) throws Exception {
    if (acceptUser(nick, passwd)) {
      try {
        preSQLDeleteUserByNick_Passwd.clearParameters();
        preSQLDeleteUserByNick_Passwd.setString(1, nick);
        preSQLDeleteUserByNick_Passwd.setString(2, passwd);
        preSQLDeleteUserByNick_Passwd.executeUpdate();
        database.commitData();
        return true;
      } catch (SQLException exc) {
        exc.printStackTrace();
      }
    } else {
      throw new Exception("Tried to delete user which doesn't exist.");
    }
    return false;
  }

 	/** Deletes an existing user from the database.
		*	@param id The <tt>userid</tt> to delete.
		*	@return <tt>true</tt> if the user exist and was successfully deleted, if user does not exists it returns <tt>false</tt>.
		*	@throws Exception if user exists, but cannot be delted from database.
		*/  
  public boolean deleteUser(int id) throws Exception {
    if (acceptUser(id)) {
      try {
        preSQLDeleteUserById.clearParameters();
        preSQLDeleteUserById.setInt(1, id);
        preSQLDeleteUserById.executeUpdate();
        database.commitData();
        return true;
      } catch (SQLException exc) {
        exc.printStackTrace();
      }
    } else {
      throw new Exception("Tried to delete user which doesn't exist.");
    }
    return false;
  }

// ------------------------------------------------------------------------------- edit User  

	/** Edits an existing user from the database.
		*	@param user The user to edit.
		*	@return If the user exist and was successfully updated the old user will be returned.
		* @throws SQLException if database cannot be reached or problems with the SQL statements.
		*	@throws Exception if user does not exists.
		*/
  public User editUser(User user) throws Exception {
    if (acceptUser(user.getId())) {

      User returnUser = getUser(user.getId());

      try {
        preSQLEditUser.clearParameters();
        preSQLEditUser.setString(1, user.getNickname() );
        preSQLEditUser.setString(2, user.getForename() );
        preSQLEditUser.setString(3, user.getSurename() );
        preSQLEditUser.setString(4, user.getEmail() );
        preSQLEditUser.setInt(5, user.getId());
        preSQLEditUser.executeUpdate();
        database.commitData();

        return returnUser;

      } catch (SQLException exc) {
        exc.printStackTrace();
        throw new SQLException("edituser(user,passwd): Failed to set userdata." + exc.getMessage());
      }
    } else {
      throw new Exception("Tried to edit user which doesn't exist.");
    }
  }

	/** Edits an existing user password from the database.
		*	@param user The user to edit.
		* @param passwd The new password.
		*	@return If the user exist and password was successfully updated the old user will be returned.
		* @throws SQLException if database cannot be reached or problems with the SQL statements.
		*	@throws Exception if user does not exists.
		*/  
  public User editUser(User user, String passwd) throws Exception{
    if (acceptUser( user.getNickname(), user.getPassword() ) ) {
      User returnUser = getUser(user.getId());
      try {
        preSQLEditUserPwd.clearParameters();
        preSQLEditUserPwd.setString(1, passwd );
        preSQLEditUserPwd.setInt(2, user.getId());
        preSQLEditUserPwd.setString(3, user.getPassword() );
        preSQLEditUserPwd.executeUpdate();
        user.setPassword(passwd);
        database.commitData();
        return returnUser;
      } catch (SQLException exc) {
        exc.printStackTrace();
        throw new SQLException("edituser(user,passwd): Failed to set userdata." + exc.getMessage());
      }
    } else {
      throw new Exception("Tried to edit user which nick and password doesn't exist.");
    }
  }

// ------------------------------------------------------------------------------- login User

	/**	Tries to login a new user.
		*	@param nick The nickname of the user who wants to login
		*	@param passwd The password of the user who wants to login
		* @return It returns the existing user object from the database if it exists, else <tt>null</tt>.
		* @throws Exception if user exists but secureIdentity could not be set.
		*/
  public User loginUser(String nick, String passwd) throws Exception {
    if (acceptUser( nick, passwd )) {
      User returnUser = getUser( nick, passwd );
      returnUser.setSecureIdentity(getRandomSecureId(10));
      try {
        preSQLSetSecureIdentity.clearParameters();
        preSQLSetSecureIdentity.setString(1, returnUser.getSecureIdentity() );
        preSQLSetSecureIdentity.setInt(2, returnUser.getId() );
        preSQLSetSecureIdentity.executeUpdate();
        database.commitData();
        return returnUser;
      } catch (SQLException exc) {
        exc.printStackTrace();
        throw new Exception("LoginUser(nick,passwd): Accepted User cannot login. Failed to set SecureIdentity.");
      }
    }
    return null;
  }

	/**	Tries to login a new user.
		*	@param nick The nickname of the user who wants to login
		*	@param passwd The password of the user who wants to login
		* @param request The HttpServletRequest.
		* @return It returns the existing user object from the database if it exists, else <tt>null</tt>.
		* @throws Exception if user exists but secureIdentity could not be set.
		*/
  public User loginUser(String nick, String passwd, HttpServletRequest request) throws Exception {
    if (acceptUser( nick, passwd )) {
      User returnUser = getUser( nick, passwd );
      returnUser.setSecureIdentity(getRandomSecureId(10));
      try {
        preSQLSetSecureIdentity.clearParameters();
        preSQLSetSecureIdentity.setString(1, returnUser.getSecureIdentity() );
        preSQLSetSecureIdentity.setInt(2, returnUser.getId() );
        preSQLSetSecureIdentity.executeUpdate();
        database.commitData();
        return returnUser;
      } catch (SQLException exc) {
        exc.printStackTrace();
        throw new Exception("LoginUser(nick,passwd,request): Accepted User cannot login. Failed to set SecureIdentity.");
      }
    }
    return null;
  }

// ------------------------------------------------------------------------------- get User

	/** Gets a user from the database.
		* @param id The id of the user to get.
		* @return The user from the database chooesen by the id, if not exists it returns <tt>null</tt>.
		* @throws SQLException if there is no conection to the database or problems with the SQL statements.
		*/
  public User getUser(int id) throws SQLException {
    User returnUser = new User();
    try {
      preSQLGetUserById.clearParameters();
      preSQLGetUserById.setInt(1, id);
      ResultSet result = preSQLGetUserById.executeQuery();
      if( result.next() ) {
        returnUser.setId( result.getInt(1));
        returnUser.setNickname( result.getString(2));
        returnUser.setPassword( result.getString(3));
        returnUser.setForename( result.getString(4));
        returnUser.setSurename( result.getString(5));
        returnUser.setEmail( result.getString(6));
        returnUser.setSecureIdentity( result.getString(7));
        returnUser.setSu_User(result.getBoolean(8));
        result.close();
        return returnUser;
      } else {
        result.close();
      }
    } catch (SQLException exc) {
      exc.printStackTrace();
      throw new SQLException("getUser(id): Failed to retrieve User-Information.\n"+exc.getMessage());
    }
    return null;
  }

	/** Gets a user from the database.
		* @param nick The nickname of the user to get.
		* @return The user from the database choosen by the nickname, if not exists it returns <tt>null</tt>.
		* @throws SQLException if there is no conection to the database or problems with the SQL statements.
		*/  
  public User getUser(String nick) throws SQLException {
    User returnUser = new User();
    try {
      preSQLGetUserByNick.clearParameters();
      preSQLGetUserByNick.setString(1, nick);
      ResultSet result = preSQLGetUserByNick.executeQuery();
      if( result.next() ) {
        returnUser.setId( result.getInt(1));
        returnUser.setNickname( result.getString(2));
        returnUser.setPassword( result.getString(3));
        returnUser.setForename( result.getString(4));
        returnUser.setSurename( result.getString(5));
        returnUser.setEmail( result.getString(6));
        returnUser.setSecureIdentity( result.getString(7));
        returnUser.setSu_User(result.getBoolean(8));
        result.close();
        return returnUser;
      } else {
        result.close();
      }
    } catch (SQLException exc) {
      exc.printStackTrace();
      throw new SQLException("getUser(nick): Failed to retrieve User-Information.\n"+exc.getMessage());
    }
    return null;
  }

	/** Gets a user from the database.
		* @param id The id of the user to get.
		* @param sid The secureIdentity of the user to get,
		* @return The user from the database choosen by the id and secureIdentity, if not exists it returns <tt>null</tt>.
		* @throws SQLException if there is no conection to the database or problems with the SQL statements.
		*/  
  public User getUser(int id, String sid) throws SQLException {
    User returnUser = new User();
    try {
      preSQLGetUserById_Sid.clearParameters();
      preSQLGetUserById_Sid.setInt(1, id);
      preSQLGetUserById_Sid.setString(2, sid);
      ResultSet result = preSQLGetUserById_Sid.executeQuery();
      if( result.next() ) {
        returnUser.setId( result.getInt(1));
        returnUser.setNickname( result.getString(2));
        returnUser.setPassword( result.getString(3));
        returnUser.setForename( result.getString(4));
        returnUser.setSurename( result.getString(5));
        returnUser.setEmail( result.getString(6));
        returnUser.setSecureIdentity( result.getString(7));
        returnUser.setSu_User(result.getBoolean(8));
        result.close();
        return returnUser;
      } else {
        result.close();
      }
    } catch (SQLException exc) {
      exc.printStackTrace();
      throw new SQLException("getUser(id,sid): Failed to retrieve User-Information.\n"+exc.getMessage());
    }
    return null;
  }
	/** Gets a user from the database.
		* @param nick The nickname of the user to get.
		* @param passwd The password of the user to get.
		* @return The user from the database choosen by the nickname and password, if not exists it returns <tt>null</tt>.
		* @throws SQLException if there is no conection to the database or problems with the SQL statements.
		*/
  public User getUser(String nick, String passwd) throws SQLException {
    User returnUser = new User();
    try {
      preSQLGetUserByNick_Passwd.clearParameters();
      preSQLGetUserByNick_Passwd.setString(1, nick);
      preSQLGetUserByNick_Passwd.setString(2, passwd);
      ResultSet result = preSQLGetUserByNick_Passwd.executeQuery();
      if( result.next() ) {
        returnUser.setId( result.getInt(1));
        returnUser.setNickname( result.getString(2));
        returnUser.setPassword( result.getString(3));
        returnUser.setForename( result.getString(4));
        returnUser.setSurename( result.getString(5));
        returnUser.setEmail( result.getString(6));
        returnUser.setSecureIdentity( result.getString(7));
        returnUser.setSu_User(result.getBoolean(8));
        result.close();
        return returnUser;
      } else {
        result.close();
      }
    } catch (SQLException exc) {
      exc.printStackTrace();
      throw new SQLException("getUser(nick,passwd): Failed to retrieve User-Information.\n"+exc.getMessage());
    }
    return null;
  }

}