/********************************************************************
  *
  *  program     : User.java
  *  author      : Helge Janicke
  *  version     : 2000-11-28(001)
  *  (c)         : Helge Janicke
  *  email       : heljanic@agent.fho-emden.de
  *  description : Class that holds information of a logged in user
  *
  *******************************************************************
  * modifications:
  * date: ******* who: ****** description: **************************
  * -----------------------------------------------------------------
  * 2000-10-19    heljanic    your description here
  * -----------------------------------------------------------------
  *******************************************************************
  * comments:
  *******************************************************************
  * license:
********************************************************************/

package de.janicke.hj.tools.servlet;

/** Class that holds the information of a user. Information can be
  * retrieved out of a Database using the UserAdministration class
  * in this package.
  */
public class User {

//--------------------------------------------- Attribute definitions

  /** Holds the nickname of the user. */
  private String nickname = "";

  /** The password of the user.
    * This should later be encrypted but in this version the
    * password is stored "as is".
    * @todo encrypt this password using JCrypt.
    */
  private String password = "";

  /** Holds the forename of the user. */
  private String forename = "";

  /** Holds the surename of the user. */
  private String surename = "";

  /** Holds the email adress of the user */
  private String email = "";

  /** String that is used to ensure the personality of the user.
    * This String is sent to and fro between the Servlet and the
    * Webbrowser.
    */
  private String secureIdentity = "";

  /** The Webbrowser used by this user. */
  private String webBrowser = "";

  /** The IP adress of the user. */
  private String ip = "";

  /** The ID number for this user in the Database. */
  private int id = -1;

  /** True if the user is logged in */
  private boolean loggedIn = false;

  /** True if the user is Superuser */
  private boolean su_flag = false;


//------------------------------------------------------ Constructors

  /** Constructs a new User object. */
  public User() {
  }


//-------------------------------------------------- Accessor methods

  /** Returns the nickname.
    * @return The nickname.
    */
  public String getNickname() {
    return this.nickname;
  }

  /** Sets the nickname.
    * @param nickname The new Nickname.
    */
  public void setNickname(String nickname) {
    this.nickname = nickname;
  }

  /** Returns the password.
    * @return The password.
    */
  public String getPassword() {
    return this.password;
  }

  /** Sets the password.
    * @param password The new Nickname.
    */
  public void setPassword(String password) {
    this.password = password;
  }

  /** Returns the forename.
    * @return The forename.
    */
  public String getForename() {
    return this.forename;
  }

  /** Sets the forename.
    * @param forename The new Forename.
    */
  public void setForename(String forename) {
    this.forename = forename;
  }

  /** Returns the surename.
    * @return The surename.
    */
  public String getSurename() {
    return this.surename;
  }

  /** Sets the surename.
    * @param surename The new Surename.
    */
  public void setSurename(String surename) {
    this.surename = surename;
  }

  /** Returns the email.
    * @return The email.
    */
  public String getEmail() {
    return this.email;
  }

  /** Sets the email.
    * @param email The new Email.
    */
  public void setEmail(String email) {
    this.email = email;
  }

  /** Returns the secureIdentity.
    * @return The secureIdentity.
    */
  public String getSecureIdentity() {
    return this.secureIdentity;
  }

  /** Sets the secureIdentity.
    * @param secureIdentity The new SecureIdentity.
    */
  public void setSecureIdentity(String secureIdentity) {
    this.secureIdentity = secureIdentity;
  }

  /** Returns the webBrowser.
    * @return The webBrowser.
    */
  public String getWebBrowser() {
    return this.webBrowser;
  }

  /** Sets the webBrowser.
    * @param webBrowser The new WebBrowser.
    */
  public void setWebBrowser(String webBrowser) {
    this.webBrowser = webBrowser;
  }

  /** Returns the IP adress of the user.
    * @return The IP.
    */
  public String getIp() {
    return this.ip;
  }

  /** Sets the IP adress of the user.
    * @param ip The new IP.
    */
  public void setIp(String ip) {
    this.ip = ip;
  }

  /** Returns the id.
    * @return The id.
    */
  public int getId() {
    return this.id;
  }

  /** Sets the id.
    * @param id The new Id.
    */
  public void setId(int id) {
    this.id = id;
  }

  /** Returns the logged in status.
    * @return true if the user is logged in, false otherwise.
    */
  public boolean isLoggedIn() {
    return this.loggedIn;
  }

/** Returns the Superuser status
  * @return true if the user is Superuser, false otherwise.
  */
  public boolean isSu_User() {
    return this.su_flag;
  }
/** Sets the Superuser status.
  * @param su_flag The new Superuser flag (true: is Superuser).
  */
  public void setSu_User(boolean su_flag) {
    this.su_flag = su_flag;
  }

  /** Sets the logged in status.
    * @param loggedIn the new logged in status (true: logged in).
    */
  public void setNickname(boolean loggedIn) {
    this.loggedIn = loggedIn;
  }

//------------------------------------------------------------ methods

  public String toString() {
    return "USER: Forename = "+getForename() + " Surename = " + getSurename() + " known as " +
         getNickname() + "\n  email: " + getEmail() + "\n [ID = " +
         getId() + "]";
  }

}